@@ -20,7 +20,7 @@
 # 
 #----------------------------------------------------------------------------
 #
-# $Id: Document.pm,v 2.62 2002/11/04 19:45:57 abw Exp $
+# $Id: Document.pm,v 2.67 2003/10/14 09:40:37 abw Exp $
 #
 #============================================================================
 
@@ -33,7 +33,7 @@ use vars qw( $VERSION $ERROR $COMPERR $DEBUG $AUTOLOAD );
 use base qw( Template::Base );
 use Template::Constants;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.62 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.67 $ =~ /(\d+)\.(\d+)/);
 
 
 #========================================================================
@@ -81,10 +81,10 @@ sub new {
 	} values %$defblocks;
 
     bless {
-	%$metadata,
-	_BLOCK     => $block,
-	_DEFBLOCKS => $defblocks,
-	_HOT       => 0,
+        %$metadata,
+        _BLOCK     => $block,
+        _DEFBLOCKS => $defblocks,
+        _HOT       => 0,
     }, $class;
 }
 
@@ -131,20 +131,22 @@ sub process {
 
     # check we're not already visiting this template
     return $context->throw(Template::Constants::ERROR_FILE, 
-			   "recursion into '$self->{ name }'")
-	if $self->{ _HOT } && ! $context->{ RECURSION };   ## RETURN ##
+                           "recursion into '$self->{ name }'")
+        if $self->{ _HOT } && ! $context->{ RECURSION };   ## RETURN ##
+
+    $context->visit($self, $defblocks);
 
-    $context->visit($defblocks);
     $self->{ _HOT } = 1;
     eval {
-	my $block = $self->{ _BLOCK };
-	$output = &$block($context);
+        my $block = $self->{ _BLOCK };
+        $output = &$block($context);
     };
     $self->{ _HOT } = 0;
+
     $context->leave();
 
     die $context->catch($@)
-	if $@;
+        if $@;
 	
     return $output;
 }
@@ -447,7 +449,7 @@ An autoload method returns METADATA items.
 
 This package subroutine is provided to effect persistance of compiled
 templates.  If the COMPILE_EXT option (to indicate a file extension
-for saving compiled templates) then the Template::Parser module call
+for saving compiled templates) then the Template::Parser module calls
 this subroutine before calling the new() constructor.  At this stage,
 the parser has a representation of the template as text strings
 containing Perl code.  We can write that to a file, enclosed in a
@@ -466,12 +468,12 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.62, distributed as part of the
-Template Toolkit version 2.09, released on 23 April 2003.
+2.67, distributed as part of the
+Template Toolkit version 2.11, released on 06 January 2004.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2002 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1996-2004 Andy Wardley.  All Rights Reserved.
   Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
